# -*- coding: utf-8 -*-
"""
Módulo que contiene clases para analizar datos de las plantas solares
instaladas en diversos estados de EEUU

@author: To-Do
@date:   sep 2019
"""

# imports
# To-Do

class Estado:
    """
    Contiene los datos de las plantas solares de un estado concreto

    Attributes:
        __nombre: el nombre del estado
        __numero_plantas: el número de plantas solares del estado
        __capacidad_instalada: la potencia instalada del estado, en MW
        __generacion_anual: valor de la energía total generada en el estado,
                            en GWh
    """

    def __init__(self, nombre: int, numero_plantas: str,
                 capacidad_instalada: str, generacion_anual: float):
        """
        Constructor que recibe como parámetros los valores iniciales de los
        atributos

        Params:
        nombre: el nombre del estado
        numero_plantas: el número de plantas solares del estado
        capacidad_instalada: la potencia instalada del estado, en MW
        generacion_anual: valor de la energía total generada en el estado,
                          en GWh
        """
        self.__nombre = nombre
        self.__numero_plantas = numero_plantas
        self.__capacidad_instalada = capacidad_instalada
        self.__generacion_anual = generacion_anual

    def get_nombre(self)-> int:
        """
        Retorna el nombre del estado
        """
        return self.__nombre

    def get_numero_plantas(self)-> str:
        """
        Retorna el número de plantas solares del estado
        """
        return self.__numero_plantas

    def get_capacidad_instalada(self)-> str:
        """
        Retorna la potencia instalada del estado, en MW
        """
        return self.__capacidad_instalada

    def get_generacion_anual(self)-> float:
        """
        Retorna el valor de la energía total generada en el estado, en GW
        """
        return self.__generacion_anual

class FicheroNoExiste(Exception):
    """
    Indica que el fichero que se quiere leer no existe
    """

class EstadoNoEncontrado(Exception):
    """
    Indica que el estado que se ha especificado no se encuentra en la lista
    """

class ListaEstados:
    """
    La clase almacena en un atributo una lista de objetos de la clase Estado.
    Dispone también de un atributo para guardar el nombre del fichero del que
    proceden los datos y otro para la fuente de los datos. También dispone de
    diversos métodos para analizar las plantas solares

    Attributes:
        __lista: una lista de objetos de la clase Estado
        __nombre_fichero: el nombre del fichero del que se leen los datos
        __fuente: fuente de los datos
    """

    def __init__(self, nombre_fichero: str):
        """
        Crea la lista de estados vacía y rellena los atributos
        nombre_fichero y fuente

        Args:
            nombre_fichero: el nombre del fichero donde se encuentran
                            los datos de las plantas solares
        Raises:
            FicheroNoExiste: si el fichero especificado no existe
        """
        # To-Do


    def lee(self):
        """
        Lee los datos de las plantas solares del fichero cuyo nombre se indica
        en el argumento nombre_fichero y los mete en la lista.
        El fichero contiene dos líneas de encabezamiento y luego los
        datos de un estado por cada línea, con sus campos (Nombre, número de
        plantas solares, capacidad instalada (MW), MW promedios por planta,
        y Generación (GWh)) separados por comas. El dato de MW promedios por
        planta se ignora.
        Se supone que los datos del fichero son correctos.

        Si el fichero no existe este método no hace nada
        """
        try:
            with open(self.__nombre_fichero, 'r') as fichero:
                # Saltamos las líneas de cabecera
                fichero.readline()
                fichero.readline()

                # Leemos el resto de las líneas
                for linea in fichero:
                    datos: List[str] = linea.split(",")
                    est = Estado(datos[0], int(datos[1]), float(datos[2]),
                                 float(datos[4]))
                    self.__lista.append(est)
        except IOError:
            pass

    def resumen(self):
        """
        Muestra en pantalla un resumen de los datos de la lista.
        """
        # Escribe el encabezamiento
        print("Datos de plantas solares, por estado. ",
              f"Número de datos {len(self.__lista)}")
        # Escribe el nombre del fichero y la fuente
        print(f"Nombre del fichero {self.__nombre_fichero}.",
              f"Fuente de los datos: {self.__fuente}")
        print(f"{'Nombre':15} {'Num Plantas':>12} {'Capacidad(MW)':>13} "+
              f"{'Generacion(GWh)':>15}")
        # Escribe los 3 primeros datos, uno por línea
        print("Tres primeros datos:")
        for est in self.__lista[0:3]:
            print(f"{est.get_nombre():15s} {est.get_numero_plantas():12d} "+
                  f"{est.get_capacidad_instalada():13.1f} "+
                  f"{est.get_generacion_anual():15.1f}")
        # Escribe los 3 últimos datos, uno por línea
        print("Tres últimos datos:")
        for est in self.__lista[-3:]:
            print(f"{est.get_nombre():15s} {est.get_numero_plantas():12d} "+
                  f"{est.get_capacidad_instalada():13.1f} "+
                  f"{est.get_generacion_anual():15.1f}")
        print()

    def eficiencia(self, estado: str)-> float:
        """
        Calcula y retorna la eficiencia de las plantas solares del estado
        cuyo nombre se pasa como parámetro

        Args:
            estado: el estado buscado
        Raises:
            EstadoNoEcontrado: si el estado no se encuentra en la lista
        """
        # To-Do


    def mas_plantas(self)-> str:
        """
        Retorna el nombre del estado que tiene mayor número de plantas solares
        o None si la lista está vacía
        """
        # To-Do


def main():
    """
    Prueba la clase ListaEstados
    """
    # To-Do



if __name__ == '__main__':
    main()
